import Tutch from 'tutch-worker';
/**
 * Wrap the CodeMirror markText functionality to work with an esprima-style SourceLocation
 */
function markText(doc, loc, options) {
    return doc.markText({ line: loc.start.line - 1, ch: loc.start.column - 1 }, { line: loc.end.line - 1, ch: loc.end.column - 1 }, options);
}
/**
 * Create a Tutch CodeMirror element.
 *
 * CodeMirror itself is a big honkin' thing. Sometimes you want to just load
 * `codemirror.js`, sometimes you want to import codemirror and have webpack
 * take care of it. I don't know which one is better, so whatever you do, just
 * pass the `CodeMirror` builder object to the function.
 *
 * The optional text is the initial program,
 */
export default function TutchCode(options) {
    const { url, host, codemirror } = options;
    const initialText = options.text || '';
    const onSuccess = options.onSuccess || (() => { });
    const onError = options.onError || (() => { });
    const onUpdate = options.onUpdate || (() => { });
    let tutch;
    let value = initialText;
    const editor = codemirror(host, {
        value,
        lineNumbers: true,
    });
    const doc = editor.getDoc();
    editor.on('update', () => {
        const text = editor.getValue();
        if (text !== value) {
            value = text;
            doc.getAllMarks().forEach(mark => mark.clear());
            tutch(text);
            onUpdate(text);
        }
    });
    tutch = Tutch({
        url,
        onSuccess: justifications => {
            justifications.forEach(justification => {
                if (justification.loc) {
                    if (justification.type === 'Justified') {
                        markText(doc, justification.loc, { className: 'TutchJustified' });
                    }
                    else {
                        markText(doc, justification.loc, { className: 'TutchNotJustified' });
                    }
                }
            });
            onSuccess(justifications);
        },
        onError: (errorMessage, loc) => {
            if (loc) {
                markText(doc, loc, { className: 'TutchSyntaxError' });
            }
            onError(errorMessage, loc);
        },
    });
    tutch(value);
}
